@@ -1,9 +1,9 @@
 /*
- * "$Id: http.h 9801 2011-05-21 04:02:41Z mike $"
+ * "$Id: http.h 10424 2012-04-23 17:26:57Z mike $"
  *
  *   Hyper-Text Transport Protocol definitions for CUPS.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -124,7 +124,7 @@ typedef enum http_auth_e		/**** HTTP authentication types ****/
   HTTP_AUTH_MD5_SESS,			/* MD5-session authentication in use */
   HTTP_AUTH_MD5_INT,			/* Digest authentication in use for body */
   HTTP_AUTH_MD5_SESS_INT,		/* MD5-session authentication in use for body */
-  HTTP_AUTH_NEGOTIATE			/* GSSAPI authentication in use @since CUPS 1.3/Mac OS X 10.5@ */
+  HTTP_AUTH_NEGOTIATE			/* GSSAPI authentication in use @since CUPS 1.3/OS X 10.5@ */
 } http_auth_t;
 
 typedef enum http_encoding_e		/**** HTTP transfer encoding values ****/
@@ -251,7 +251,7 @@ typedef enum http_status_e		/**** HTTP status codes ****/
   HTTP_NOT_SUPPORTED,			/* HTTP version not supported */
 
   HTTP_AUTHORIZATION_CANCELED = 1000,	/* User canceled authorization @since CUPS 1.4@ */
-  HTTP_PKI_ERROR,			/* Error negotiating a secure connection @since CUPS 1.5/Mac OS X 10.7@ */
+  HTTP_PKI_ERROR,			/* Error negotiating a secure connection @since CUPS 1.5/OS X 10.7@ */
   HTTP_WEBIF_DISABLED			/* Web interface is disabled @private@ */
 } http_status_t;
 
@@ -292,7 +292,7 @@ typedef enum http_version_e		/**** HTTP version numbers ****/
 typedef union _http_addr_u		/**** Socket address union, which
 					 **** makes using IPv6 and other
 					 **** address types easier and
-					 **** more portable. @since CUPS 1.2/Mac OS X 10.5@
+					 **** more portable. @since CUPS 1.2/OS X 10.5@
 					 ****/
 {
   struct sockaddr	addr;		/* Base structure for family value */
@@ -309,7 +309,7 @@ typedef union _http_addr_u		/**** Socket address union, which
 typedef struct http_addrlist_s		/**** Socket address list, which is
 					 **** used to enumerate all of the
 					 **** addresses that are associated
-					 **** with a hostname. @since CUPS 1.2/Mac OS X 10.5@
+					 **** with a hostname. @since CUPS 1.2/OS X 10.5@
 					 ****/
 {
   struct http_addrlist_s *next;		/* Pointer to next address in list */
@@ -318,14 +318,14 @@ typedef struct http_addrlist_s		/**** Socket address list, which is
 
 typedef struct _http_s http_t;		/**** HTTP connection type ****/
 
-typedef struct http_credential_s	/**** HTTP credential data @since CUPS 1.5/Mac OS X 10.7@ ****/
+typedef struct http_credential_s	/**** HTTP credential data @since CUPS 1.5/OS X 10.7@ ****/
 {
   void		*data;			/* Pointer to credential data */
   size_t	datalen;		/* Credential length */
 } http_credential_t;
 
 typedef int (*http_timeout_cb_t)(http_t *http, void *user_data);
-					/**** HTTP timeout callback @since CUPS 1.5/Mac OS X 10.7@ ****/
+					/**** HTTP timeout callback @since CUPS 1.5/OS X 10.7@ ****/
 
 
 
@@ -357,10 +357,7 @@ extern void		httpInitialize(void);
 extern int		httpOptions(http_t *http, const char *uri);
 extern int		httpPost(http_t *http, const char *uri);
 extern int		httpPrintf(http_t *http, const char *format, ...)
-#  ifdef __GNUC__
-__attribute__ ((__format__ (__printf__, 2, 3)))
-#  endif /* __GNUC__ */
-;
+			__attribute__ ((__format__ (__printf__, 2, 3)));
 extern int		httpPut(http_t *http, const char *uri);
 extern int		httpRead(http_t *http, char *buffer, int length) _CUPS_DEPRECATED;
 extern int		httpReconnect(http_t *http);
@@ -398,7 +395,7 @@ extern void		httpSeparate2(const char *uri,
 				      char *host, int hostlen, int *port,
 				      char *resource, int resourcelen) _CUPS_DEPRECATED;
 
-/**** New in CUPS 1.2/Mac OS X 10.5 ****/
+/**** New in CUPS 1.2/OS X 10.5 ****/
 extern int		httpAddrAny(const http_addr_t *addr) _CUPS_API_1_2;
 extern http_addrlist_t	*httpAddrConnect(http_addrlist_t *addrlist, int *sock) _CUPS_API_1_2;
 extern int		httpAddrEqual(const http_addr_t *addr1,
@@ -446,12 +443,12 @@ extern void		httpSetLength(http_t *http, size_t length) _CUPS_API_1_2;
 extern ssize_t		httpWrite2(http_t *http, const char *buffer,
 			           size_t length) _CUPS_API_1_2;
 
-/**** New in CUPS 1.3/Mac OS X 10.5 ****/
+/**** New in CUPS 1.3/OS X 10.5 ****/
 extern char		*httpGetAuthString(http_t *http) _CUPS_API_1_3;
 extern void		httpSetAuthString(http_t *http, const char *scheme,
 			                  const char *data) _CUPS_API_1_3;
 
-/**** New in CUPS 1.5/Mac OS X 10.7 ****/
+/**** New in CUPS 1.5/OS X 10.7 ****/
 extern int		httpAddCredential(cups_array_t *credentials,
 			                  const void *data, size_t datalen)
 					  _CUPS_API_1_5;
@@ -462,7 +459,17 @@ extern void		httpFreeCredentials(cups_array_t *certs) _CUPS_API_1_5;
 extern int		httpSetCredentials(http_t *http, cups_array_t *certs)
 					   _CUPS_API_1_5;
 extern void		httpSetTimeout(http_t *http, double timeout,
-			               http_timeout_cb_t cb, void *user_data);
+			               http_timeout_cb_t cb, void *user_data)
+			               _CUPS_API_1_5;
+
+/**** New in CUPS 1.6/OS X 10.8 ****/
+extern http_addrlist_t	*httpAddrConnect2(http_addrlist_t *addrlist, int *sock,
+			                  int msec, int *cancel)
+			                  _CUPS_API_1_6;
+extern http_state_t	httpGetState(http_t *http) _CUPS_API_1_6;
+extern http_version_t	httpGetVersion(http_t *http) _CUPS_API_1_6;
+extern int		httpReconnect2(http_t *http, int msec, int *cancel)
+			               _CUPS_API_1_6;
 
 
 /*
@@ -475,5 +482,5 @@ extern void		httpSetTimeout(http_t *http, double timeout,
 #endif /* !_CUPS_HTTP_H_ */
 
 /*
- * End of "$Id: http.h 9801 2011-05-21 04:02:41Z mike $".
+ * End of "$Id: http.h 10424 2012-04-23 17:26:57Z mike $".
  */
