@@ -1,9 +1,9 @@
 /*
- * "$Id: ipp-support.c 9793 2011-05-20 03:49:49Z mike $"
+ * "$Id: ipp-support.c 10424 2012-04-23 17:26:57Z mike $"
  *
  *   Internet Printing Protocol support functions for CUPS.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -16,16 +16,19 @@
  *
  * Contents:
  *
- *   _ippAttrString() - Convert the attribute's value to a string.
- *   ippErrorString() - Return a name for the given status code.
- *   ippErrorValue()  - Return a status code for the given name.
- *   ippOpString()    - Return a name for the given operation id.
- *   ippOpValue()     - Return an operation id for the given name.
- *   ippPort()        - Return the default IPP port number.
- *   ippSetPort()     - Set the default port number.
- *   ippTagString()   - Return the tag name corresponding to a tag value.
- *   ippTagValue()    - Return the tag value corresponding to a tag name.
- *   ipp_col_string() - Convert a collection to a string.
+ *   ippAttributeString() - Convert the attribute's value to a string.
+ *   ippEnumString()	  - Return a string corresponding to the enum value.
+ *   ippEnumValue()	  - Return the value associated with a given enum
+ *			    string.
+ *   ippErrorString()	  - Return a name for the given status code.
+ *   ippErrorValue()	  - Return a status code for the given name.
+ *   ippOpString()	  - Return a name for the given operation id.
+ *   ippOpValue()	  - Return an operation id for the given name.
+ *   ippPort()		  - Return the default IPP port number.
+ *   ippSetPort()	  - Set the default port number.
+ *   ippTagString()	  - Return the tag name corresponding to a tag value.
+ *   ippTagValue()	  - Return the tag value corresponding to a tag name.
+ *   ipp_col_string()	  - Convert a collection to a string.
  */
 
 /*
@@ -40,18 +43,18 @@
  */
 
 static const char * const ipp_status_oks[] =	/* "OK" status codes */
-		{
+		{				/* (name) = abandoned standard value */
 		  "successful-ok",
 		  "successful-ok-ignored-or-substituted-attributes",
 		  "successful-ok-conflicting-attributes",
 		  "successful-ok-ignored-subscriptions",
-		  "successful-ok-ignored-notifications",
+		  "(successful-ok-ignored-notifications)",
 		  "successful-ok-too-many-events",
-		  "successful-ok-but-cancel-subscription",
+		  "(successful-ok-but-cancel-subscription)",
 		  "successful-ok-events-complete"
 		},
 		* const ipp_status_400s[] =	/* Client errors */
-		{
+		{				/* (name) = abandoned standard value */
 		  "client-error-bad-request",
 		  "client-error-forbidden",
 		  "client-error-not-authenticated",
@@ -74,8 +77,12 @@ static const char * const ipp_status_oks[] =	/* "OK" status codes */
 		  "client-error-attributes-not-settable",
 		  "client-error-ignored-all-subscriptions",
 		  "client-error-too-many-subscriptions",
-		  "client-error-ignored-all-notifications",
-		  "client-error-print-support-file-not-found"
+		  "(client-error-ignored-all-notifications)",
+		  "(client-error-client-print-support-file-not-found)",
+		  "client-error-document-password-error",
+		  "client-error-document-permission-error",
+		  "client-error-document-security-error",
+		  "client-error-document-unprintable-error"
 		},
 		* const ipp_status_500s[] =		/* Server errors */
 		{
@@ -89,7 +96,9 @@ static const char * const ipp_status_oks[] =	/* "OK" status codes */
 		  "server-error-busy",
 		  "server-error-job-canceled",
 		  "server-error-multiple-document-jobs-not-supported",
-		  "server-error-printer-is-deactivated"
+		  "server-error-printer-is-deactivated",
+		  "server-error-too-many-jobs",
+		  "server-error-too-many-documents"
 		},
 		* const ipp_status_1000s[] =		/* CUPS internal */
 		{
@@ -97,11 +106,11 @@ static const char * const ipp_status_oks[] =	/* "OK" status codes */
 		  "cups-pki-error",
 		  "cups-upgrade-required"
 		};
-static char	* const ipp_std_ops[] =
+static const char * const ipp_std_ops[] =
 		{
 		  /* 0x0000 - 0x000f */
-		  "unknown-00",
-		  "unknown-01",
+		  "0x00",
+		  "0x01",
 		  "Print-Job",
 		  "Print-URI",
 		  "Validate-Job",
@@ -115,7 +124,7 @@ static char	* const ipp_std_ops[] =
 		  "Hold-Job",
 		  "Release-Job",
 		  "Restart-Job",
-		  "unknown-0f",
+		  "0x0f",
 
 		  /* 0x0010 - 0x001f */
 		  "Pause-Printer",
@@ -131,13 +140,13 @@ static char	* const ipp_std_ops[] =
 		  "Renew-Subscription",
 		  "Cancel-Subscription",
 		  "Get-Notifications",
-		  "Send-Notifications",
-		  "unknown-1e",
-		  "unknown-1f",
+		  "(Send-Notifications)",
+		  "(Get-Resource-Attributes)",
+		  "(Get-Resource-Data)",
 
 		  /* 0x0020 - 0x002f */
-		  "unknown-20",
-		  "Get-Printer-Support-Files",
+		  "(Get-Resources)",
+		  "(Get-Printer-Support-Files)",
 		  "Enable-Printer",
 		  "Disable-Printer",
 		  "Pause-Printer-After-Current-Job",
@@ -153,10 +162,10 @@ static char	* const ipp_std_ops[] =
 		  "Suspend-Current-Job",
 		  "Resume-Job",
 
-		  /* 0x0030 - 0x003b */
+		  /* 0x0030 - 0x003d */
 		  "Promote-Job",
 		  "Schedule-Job-After",
-		  "unknown-32",
+		  "0x32",
 		  "Cancel-Document",
 		  "Get-Document-Attributes",
 		  "Get-Documents",
@@ -165,7 +174,9 @@ static char	* const ipp_std_ops[] =
 		  "Cancel-Jobs",
 		  "Cancel-My-Jobs",
 		  "Resubmit-Job",
-		  "Close-Job"
+		  "Close-Job",
+		  "Identify-Printer",
+		  "Validate-Document"
 		},
 		* const ipp_cups_ops[] =
 		{
@@ -205,46 +216,48 @@ static char	* const ipp_std_ops[] =
 					/* 0x06 */
 		  "event-notification-attributes-tag",
 					/* 0x07 */
-		  "unknown-08",		/* 0x08 */
-		  "unknown-09",		/* 0x09 */
-		  "unknown-0a",		/* 0x0a */
-		  "unknown-0b",		/* 0x0b */
-		  "unknown-0c",		/* 0x0c */
-		  "unknown-0d",		/* 0x0d */
-		  "unknown-0e",		/* 0x0e */
-		  "unknown-0f",		/* 0x0f */
+		  "(resource-attributes-tag)",
+		  			/* 0x08 */
+		  "document-attributes-tag",
+					/* 0x09 */
+		  "0x0a",		/* 0x0a */
+		  "0x0b",		/* 0x0b */
+		  "0x0c",		/* 0x0c */
+		  "0x0d",		/* 0x0d */
+		  "0x0e",		/* 0x0e */
+		  "0x0f",		/* 0x0f */
 		  "unsupported",	/* 0x10 */
 		  "default",		/* 0x11 */
 		  "unknown",		/* 0x12 */
 		  "no-value",		/* 0x13 */
-		  "unknown-14",		/* 0x14 */
+		  "0x14",		/* 0x14 */
 		  "not-settable",	/* 0x15 */
 		  "delete-attribute",	/* 0x16 */
 		  "admin-define",	/* 0x17 */
-		  "unknown-18",		/* 0x18 */
-		  "unknown-19",		/* 0x19 */
-		  "unknown-1a",		/* 0x1a */
-		  "unknown-1b",		/* 0x1b */
-		  "unknown-1c",		/* 0x1c */
-		  "unknown-1d",		/* 0x1d */
-		  "unknown-1e",		/* 0x1e */
-		  "unknown-1f",		/* 0x1f */
-		  "unknown-20",		/* 0x20 */
+		  "0x18",		/* 0x18 */
+		  "0x19",		/* 0x19 */
+		  "0x1a",		/* 0x1a */
+		  "0x1b",		/* 0x1b */
+		  "0x1c",		/* 0x1c */
+		  "0x1d",		/* 0x1d */
+		  "0x1e",		/* 0x1e */
+		  "0x1f",		/* 0x1f */
+		  "0x20",		/* 0x20 */
 		  "integer",		/* 0x21 */
 		  "boolean",		/* 0x22 */
 		  "enum",		/* 0x23 */
-		  "unknown-24",		/* 0x24 */
-		  "unknown-25",		/* 0x25 */
-		  "unknown-26",		/* 0x26 */
-		  "unknown-27",		/* 0x27 */
-		  "unknown-28",		/* 0x28 */
-		  "unknown-29",		/* 0x29 */
-		  "unknown-2a",		/* 0x2a */
-		  "unknown-2b",		/* 0x2b */
-		  "unknown-2c",		/* 0x2c */
-		  "unknown-2d",		/* 0x2d */
-		  "unknown-2e",		/* 0x2e */
-		  "unknown-2f",		/* 0x2f */
+		  "0x24",		/* 0x24 */
+		  "0x25",		/* 0x25 */
+		  "0x26",		/* 0x26 */
+		  "0x27",		/* 0x27 */
+		  "0x28",		/* 0x28 */
+		  "0x29",		/* 0x29 */
+		  "0x2a",		/* 0x2a */
+		  "0x2b",		/* 0x2b */
+		  "0x2c",		/* 0x2c */
+		  "0x2d",		/* 0x2d */
+		  "0x2e",		/* 0x2e */
+		  "0x2f",		/* 0x2f */
 		  "octetString",	/* 0x30 */
 		  "dateTime",		/* 0x31 */
 		  "resolution",		/* 0x32 */
@@ -253,18 +266,18 @@ static char	* const ipp_std_ops[] =
 		  "textWithLanguage",	/* 0x35 */
 		  "nameWithLanguage",	/* 0x36 */
 		  "endCollection",	/* 0x37 */
-		  "unknown-38",		/* 0x38 */
-		  "unknown-39",		/* 0x39 */
-		  "unknown-3a",		/* 0x3a */
-		  "unknown-3b",		/* 0x3b */
-		  "unknown-3c",		/* 0x3c */
-		  "unknown-3d",		/* 0x3d */
-		  "unknown-3e",		/* 0x3e */
-		  "unknown-3f",		/* 0x3f */
-		  "unknown-40",		/* 0x40 */
+		  "0x38",		/* 0x38 */
+		  "0x39",		/* 0x39 */
+		  "0x3a",		/* 0x3a */
+		  "0x3b",		/* 0x3b */
+		  "0x3c",		/* 0x3c */
+		  "0x3d",		/* 0x3d */
+		  "0x3e",		/* 0x3e */
+		  "0x3f",		/* 0x3f */
+		  "0x40",		/* 0x40 */
 		  "textWithoutLanguage",/* 0x41 */
 		  "nameWithoutLanguage",/* 0x42 */
-		  "unknown-43",		/* 0x43 */
+		  "0x43",		/* 0x43 */
 		  "keyword",		/* 0x44 */
 		  "uri",		/* 0x45 */
 		  "uriScheme",		/* 0x46 */
@@ -273,22 +286,115 @@ static char	* const ipp_std_ops[] =
 		  "mimeMediaType",	/* 0x49 */
 		  "memberAttrName"	/* 0x4a */
 		};
-static const char * const job_states[] =
-{					/* job-state enums */
-  "pending",
-  "pending-held",
-  "processing",
-  "processing-stopped",
-  "canceled",
-  "aborted",
-  "completed"
-};
-static const char * const printer_states[] =
-{					/* printer-state enums */
-  "idle",
-  "processing",
-  "stopped",
-};
+static const char * const ipp_document_states[] =
+		{			/* document-state-enums */
+		  "pending",
+		  "4",
+		  "processing",
+		  "6",
+		  "canceled",
+		  "aborted",
+		  "completed"
+		},
+		* const ipp_finishings[] =
+		{			/* finishings enums */
+		  "none",
+		  "staple",
+		  "punch",
+		  "cover",
+		  "bind",
+		  "saddle-stitch",
+		  "edge-stitch",
+		  "fold",
+		  "trim",
+		  "bale",
+		  "booklet-maker",
+		  "jog-offset",
+		  "15",
+		  "16",
+		  "17",
+		  "18",
+		  "19",
+		  "staple-top-left",
+		  "staple-bottom-left",
+		  "staple-top-right",
+		  "staple-bottom-right",
+		  "edge-stitch-left",
+		  "edge-stitch-top",
+		  "edge-stitch-right",
+		  "edge-stitch-bottom",
+		  "staple-dual-left",
+		  "staple-dual-top",
+		  "staple-dual-right",
+		  "staple-dual-bottom",
+		  "32",
+		  "33",
+		  "34",
+		  "35",
+		  "36",
+		  "37",
+		  "38",
+		  "39",
+		  "40",
+		  "41",
+		  "42",
+		  "43",
+		  "44",
+		  "45",
+		  "46",
+		  "47",
+		  "48",
+		  "49",
+		  "bind-left",
+		  "bind-top",
+		  "bind-right",
+		  "bind-bottom",
+		  "54",
+		  "55",
+		  "56",
+		  "57",
+		  "58",
+		  "59",
+		  "trim-after-pages",
+		  "trim-after-documents",
+		  "trim-after-copies",
+		  "trim-after-job"
+		},
+		* const ipp_job_collation_types[] =
+		{			/* job-collation-type enums */
+		  "uncollated-sheets",
+		  "collated-documents",
+		  "uncollated-documents"
+		},
+		* const ipp_job_states[] =
+		{			/* job-state enums */
+		  "pending",
+		  "pending-held",
+		  "processing",
+		  "processing-stopped",
+		  "canceled",
+		  "aborted",
+		  "completed"
+		},
+		* const ipp_orientation_requesteds[] =
+		{			/* orientation-requested enums */
+		  "portrait",
+		  "landscape",
+		  "reverse-landscape",
+		  "reverse-portrait"
+		},
+		* const ipp_print_qualities[] =
+		{			/* print-quality enums */
+		  "draft",
+		  "normal",
+		  "high"
+		},
+		* const ipp_printer_states[] =
+		{			/* printer-state enums */
+		  "idle",
+		  "processing",
+		  "stopped",
+		};
 
 
 /*
@@ -299,24 +405,27 @@ static size_t	ipp_col_string(ipp_t *col, char *buffer, size_t bufsize);
 
 
 /*
- * '_ippAttrString()' - Convert the attribute's value to a string.
+ * 'ippAttributeString()' - Convert the attribute's value to a string.
  *
  * Returns the number of bytes that would be written, not including the
  * trailing nul. The buffer pointer can be NULL to get the required length,
  * just like (v)snprintf.
+ *
+ * @since CUPS 1.6/OS X 10.8@
  */
 
 size_t					/* O - Number of bytes less nul */
-_ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
-               char            *buffer,	/* I - String buffer or NULL */
-               size_t          bufsize)	/* I - Size of string buffer */
+ippAttributeString(
+    ipp_attribute_t *attr,		/* I - Attribute */
+    char            *buffer,		/* I - String buffer or NULL */
+    size_t          bufsize)		/* I - Size of string buffer */
 {
   int		i;			/* Looping var */
   char		*bufptr,		/* Pointer into buffer */
 		*bufend,		/* End of buffer */
 		temp[256];		/* Temporary string */
   const char	*ptr;			/* Pointer into string */
-  ipp_value_t	*val;			/* Current value */
+  _ipp_value_t	*val;			/* Current value */
 
 
   if (!attr || !attr->name)
@@ -346,30 +455,13 @@ _ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
     switch (attr->value_tag & ~IPP_TAG_COPY)
     {
       case IPP_TAG_ENUM :
-          if (!strcmp(attr->name, "printer-state") &&
-              val->integer >= IPP_PRINTER_IDLE &&
-              val->integer <= IPP_PRINTER_STOPPED)
-          {
-            ptr = printer_states[val->integer - IPP_PRINTER_IDLE];
+          ptr = ippEnumString(attr->name, val->integer);
 
-            if (buffer && bufptr < bufend)
-              strlcpy(bufptr, ptr, bufend - bufptr + 1);
+          if (buffer && bufptr < bufend)
+            strlcpy(bufptr, ptr, bufend - bufptr + 1);
 
-            bufptr += strlen(ptr);
-            break;
-          }
-          else if (!strcmp(attr->name, "job-state") &&
-		   val->integer >= IPP_JOB_PENDING &&
-		   val->integer <= IPP_JOB_COMPLETED)
-          {
-            ptr = job_states[val->integer - IPP_JOB_PENDING];
-
-            if (buffer && bufptr < bufend)
-              strlcpy(bufptr, ptr, bufend - bufptr + 1);
-
-            bufptr += strlen(ptr);
-            break;
-          }
+          bufptr += strlen(ptr);
+          break;
 
       case IPP_TAG_INTEGER :
           if (buffer && bufptr < bufend)
@@ -400,12 +492,12 @@ _ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
             bufptr += snprintf(bufptr, bufend - bufptr + 1, "%dx%d%s",
                                val->resolution.xres, val->resolution.yres,
                                val->resolution.units == IPP_RES_PER_INCH ?
-                                   "dpi" : "dpc");
+                                   "dpi" : "dpcm");
           else
             bufptr += snprintf(temp, sizeof(temp), "%dx%d%s",
                                val->resolution.xres, val->resolution.yres,
                                val->resolution.units == IPP_RES_PER_INCH ?
-                                   "dpi" : "dpc");
+                                   "dpi" : "dpcm");
           break;
 
       case IPP_TAG_DATE :
@@ -437,6 +529,7 @@ _ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
       case IPP_TAG_KEYWORD :
       case IPP_TAG_CHARSET :
       case IPP_TAG_URI :
+      case IPP_TAG_URISCHEME :
       case IPP_TAG_MIMETYPE :
       case IPP_TAG_LANGUAGE :
       case IPP_TAG_TEXTLANG :
@@ -446,7 +539,7 @@ _ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
 
           for (ptr = val->string.text; *ptr; ptr ++)
           {
-            if (*ptr == '\\' || *ptr == '\"')
+            if (*ptr == '\\' || *ptr == '\"' || *ptr == '[')
             {
               if (buffer && bufptr < bufend)
                 *bufptr = '\\';
@@ -457,6 +550,25 @@ _ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
               *bufptr = *ptr;
             bufptr ++;
           }
+
+          if (val->string.language)
+          {
+           /*
+            * Add "[language]" to end of string...
+            */
+
+            if (buffer && bufptr < bufend)
+              *bufptr = '[';
+            bufptr ++;
+
+            if (buffer && bufptr < bufend)
+              strlcpy(bufptr, val->string.language, bufend - bufptr);
+            bufptr += strlen(val->string.language);
+
+            if (buffer && bufptr < bufend)
+              *bufptr = ']';
+            bufptr ++;
+          }
           break;
 
       case IPP_TAG_BEGIN_COLLECTION :
@@ -516,6 +628,159 @@ _ippAttrString(ipp_attribute_t *attr,	/* I - Attribute */
 }
 
 
+/*
+ * 'ippEnumString()' - Return a string corresponding to the enum value.
+ */
+
+const char *				/* O - Enum string */
+ippEnumString(const char *attrname,	/* I - Attribute name */
+              int        enumvalue)	/* I - Enum value */
+{
+  _cups_globals_t *cg = _cupsGlobals();	/* Pointer to library globals */
+
+
+ /*
+  * Check for standard enum values...
+  */
+
+  if (!strcmp(attrname, "document-state") &&
+      enumvalue >= 3 &&
+      enumvalue <= (3 + (int)(sizeof(ipp_document_states) /
+                              sizeof(ipp_document_states[0]))))
+    return (ipp_document_states[enumvalue - 3]);
+  else if ((!strcmp(attrname, "finishings") ||
+            !strcmp(attrname, "finishings-actual") ||
+            !strcmp(attrname, "finishings-default") ||
+            !strcmp(attrname, "finishings-ready") ||
+            !strcmp(attrname, "finishings-supported")) &&
+           enumvalue >= 3 &&
+           enumvalue <= (3 + (int)(sizeof(ipp_finishings) / sizeof(ipp_finishings[0]))))
+    return (ipp_finishings[enumvalue - 3]);
+  else if ((!strcmp(attrname, "job-collation-type") ||
+            !strcmp(attrname, "job-collation-type-actual")) &&
+           enumvalue >= 3 &&
+           enumvalue <= (3 + (int)(sizeof(ipp_job_collation_types) /
+                                   sizeof(ipp_job_collation_types[0]))))
+    return (ipp_job_collation_types[enumvalue - 3]);
+  else if (!strcmp(attrname, "job-state") &&
+	   enumvalue >= IPP_JOB_PENDING && enumvalue <= IPP_JOB_COMPLETED)
+    return (ipp_job_states[enumvalue - IPP_JOB_PENDING]);
+  else if (!strcmp(attrname, "operations-supported"))
+    return (ippOpString((ipp_op_t)enumvalue));
+  else if ((!strcmp(attrname, "orientation-requested") ||
+            !strcmp(attrname, "orientation-requested-actual") ||
+            !strcmp(attrname, "orientation-requested-default") ||
+            !strcmp(attrname, "orientation-requested-supported")) &&
+           enumvalue >= 3 &&
+           enumvalue <= (3 + (int)(sizeof(ipp_orientation_requesteds) /
+                                   sizeof(ipp_orientation_requesteds[0]))))
+    return (ipp_orientation_requesteds[enumvalue - 3]);
+  else if ((!strcmp(attrname, "print-quality") ||
+            !strcmp(attrname, "print-quality-actual") ||
+            !strcmp(attrname, "print-quality-default") ||
+            !strcmp(attrname, "print-quality-supported")) &&
+           enumvalue >= 3 &&
+           enumvalue <= (3 + (int)(sizeof(ipp_print_qualities) /
+                                   sizeof(ipp_print_qualities[0]))))
+    return (ipp_print_qualities[enumvalue - 3]);
+  else if (!strcmp(attrname, "printer-state") &&
+           enumvalue >= IPP_PRINTER_IDLE && enumvalue <= IPP_PRINTER_STOPPED)
+    return (ipp_printer_states[enumvalue - IPP_PRINTER_IDLE]);
+
+ /*
+  * Not a standard enum value, just return the decimal equivalent...
+  */
+
+  snprintf(cg->ipp_unknown, sizeof(cg->ipp_unknown), "%d", enumvalue);
+  return (cg->ipp_unknown);
+}
+
+
+/*
+ * 'ippEnumValue()' - Return the value associated with a given enum string.
+ */
+
+int					/* O - Enum value or -1 if unknown */
+ippEnumValue(const char *attrname,	/* I - Attribute name */
+             const char *enumstring)	/* I - Enum string */
+{
+  int		i,			/* Looping var */
+		num_strings;		/* Number of strings to compare */
+  const char * const *strings;		/* Strings to compare */
+
+
+ /*
+  * If the string is just a number, return it...
+  */
+
+  if (isdigit(*enumstring & 255))
+    return (strtol(enumstring, NULL, 0));
+
+ /*
+  * Otherwise look up the string...
+  */
+
+  if (!strcmp(attrname, "document-state"))
+  {
+    num_strings = (int)(sizeof(ipp_document_states) / sizeof(ipp_document_states[0]));
+    strings     = ipp_document_states;
+  }
+  else if (!strcmp(attrname, "finishings") ||
+	   !strcmp(attrname, "finishings-actual") ||
+	   !strcmp(attrname, "finishings-default") ||
+	   !strcmp(attrname, "finishings-ready") ||
+	   !strcmp(attrname, "finishings-supported"))
+  {
+    num_strings = (int)(sizeof(ipp_finishings) / sizeof(ipp_finishings[0]));
+    strings     = ipp_finishings;
+  }
+  else if (!strcmp(attrname, "job-collation-type") ||
+           !strcmp(attrname, "job-collation-type-actual"))
+  {
+    num_strings = (int)(sizeof(ipp_job_collation_types) /
+                        sizeof(ipp_job_collation_types[0]));
+    strings     = ipp_job_collation_types;
+  }
+  else if (!strcmp(attrname, "job-state"))
+  {
+    num_strings = (int)(sizeof(ipp_job_states) / sizeof(ipp_job_states[0]));
+    strings     = ipp_job_states;
+  }
+  else if (!strcmp(attrname, "operations-supported"))
+    return (ippOpValue(enumstring));
+  else if (!strcmp(attrname, "orientation-requested") ||
+           !strcmp(attrname, "orientation-requested-actual") ||
+           !strcmp(attrname, "orientation-requested-default") ||
+           !strcmp(attrname, "orientation-requested-supported"))
+  {
+    num_strings = (int)(sizeof(ipp_orientation_requesteds) /
+                        sizeof(ipp_orientation_requesteds[0]));
+    strings     = ipp_orientation_requesteds;
+  }
+  else if (!strcmp(attrname, "print-quality") ||
+           !strcmp(attrname, "print-quality-actual") ||
+           !strcmp(attrname, "print-quality-default") ||
+           !strcmp(attrname, "print-quality-supported"))
+  {
+    num_strings = (int)(sizeof(ipp_print_qualities) / sizeof(ipp_print_qualities[0]));
+    strings     = ipp_print_qualities;
+  }
+  else if (!strcmp(attrname, "printer-state"))
+  {
+    num_strings = (int)(sizeof(ipp_printer_states) / sizeof(ipp_printer_states[0]));
+    strings     = ipp_printer_states;
+  }
+  else
+    return (-1);
+
+  for (i = 0; i < num_strings; i ++)
+    if (!strcmp(enumstring, strings[i]))
+      return (i + 3);
+
+  return (-1);
+}
+
+
 /*
  * 'ippErrorString()' - Return a name for the given status code.
  */
@@ -544,10 +809,10 @@ ippErrorString(ipp_status_t error)	/* I - Error status */
     return (ipp_status_1000s[error - IPP_AUTHENTICATION_CANCELED]);
 
  /*
-  * No, build an "unknown-xxxx" error string...
+  * No, build an "0xxxxx" error string...
   */
 
-  sprintf(cg->ipp_unknown, "unknown-%04x", error);
+  sprintf(cg->ipp_unknown, "0x%04x", error);
 
   return (cg->ipp_unknown);
 }
@@ -556,7 +821,7 @@ ippErrorString(ipp_status_t error)	/* I - Error status */
 /*
  * 'ippErrorValue()' - Return a status code for the given name.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 ipp_status_t				/* O - IPP status code */
@@ -594,7 +859,7 @@ ippErrorValue(const char *name)		/* I - Name */
 /*
  * 'ippOpString()' - Return a name for the given operation id.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 const char *				/* O - Name */
@@ -617,10 +882,10 @@ ippOpString(ipp_op_t op)		/* I - Operation ID */
     return (ipp_cups_ops2[0]);
 
  /*
-  * No, build an "unknown-xxxx" operation string...
+  * No, build an "0xxxxx" operation string...
   */
 
-  sprintf(cg->ipp_unknown, "unknown-%04x", op);
+  sprintf(cg->ipp_unknown, "0x%04x", op);
 
   return (cg->ipp_unknown);
 }
@@ -629,7 +894,7 @@ ippOpString(ipp_op_t op)		/* I - Operation ID */
 /*
  * 'ippOpValue()' - Return an operation id for the given name.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 ipp_op_t				/* O - Operation ID */
@@ -638,6 +903,9 @@ ippOpValue(const char *name)		/* I - Textual name */
   int		i;
 
 
+  if (!strncmp(name, "0x", 2))
+    return ((ipp_op_t)strtol(name + 2, NULL, 16));
+
   for (i = 0; i < (sizeof(ipp_std_ops) / sizeof(ipp_std_ops[0])); i ++)
     if (!_cups_strcasecmp(name, ipp_std_ops[i]))
       return ((ipp_op_t)i);
@@ -702,7 +970,7 @@ ippSetPort(int p)			/* I - Port number to use */
  *
  * The returned names are defined in RFC 2911 and 3382.
  *
- * @since CUPS 1.4/Mac OS X 10.6@
+ * @since CUPS 1.4/OS X 10.6@
  */
 
 const char *				/* O - Tag name */
@@ -722,7 +990,7 @@ ippTagString(ipp_tag_t tag)		/* I - Tag value */
  *
  * The tag names are defined in RFC 2911 and 3382.
  *
- * @since CUPS 1.4/Mac OS X 10.6@
+ * @since CUPS 1.4/OS X 10.6@
  */
 
 ipp_tag_t				/* O - Tag value */
@@ -773,6 +1041,7 @@ ipp_col_string(ipp_t  *col,		/* I - Collection attribute */
 {
   char			*bufptr,	/* Position in buffer */
 			*bufend,	/* End of buffer */
+			prefix = '{',	/* Prefix character */
 			temp[256];	/* Temporary string */
   ipp_attribute_t	*attr;		/* Current member attribute */
 
@@ -780,24 +1049,32 @@ ipp_col_string(ipp_t  *col,		/* I - Collection attribute */
   bufptr = buffer;
   bufend = buffer + bufsize - 1;
 
-  if (buffer && bufptr < bufend)
-    *bufptr = '{';
-  bufptr ++;
-
   for (attr = col->attrs; attr; attr = attr->next)
   {
     if (!attr->name)
       continue;
 
+    if (buffer && bufptr < bufend)
+      *bufptr = prefix;
+    bufptr ++;
+    prefix = ' ';
+
     if (buffer && bufptr < bufend)
       bufptr += snprintf(bufptr, bufend - bufptr + 1, "%s=", attr->name);
     else
       bufptr += strlen(attr->name) + 1;
 
     if (buffer && bufptr < bufend)
-      bufptr += _ippAttrString(attr, bufptr, bufend - bufptr + 1);
+      bufptr += ippAttributeString(attr, bufptr, bufend - bufptr + 1);
     else
-      bufptr += _ippAttrString(attr, temp, sizeof(temp));
+      bufptr += ippAttributeString(attr, temp, sizeof(temp));
+  }
+
+  if (prefix == '{')
+  {
+    if (buffer && bufptr < bufend)
+      *bufptr = prefix;
+    bufptr ++;
   }
 
   if (buffer && bufptr < bufend)
@@ -809,5 +1086,5 @@ ipp_col_string(ipp_t  *col,		/* I - Collection attribute */
 
 
 /*
- * End of "$Id: ipp-support.c 9793 2011-05-20 03:49:49Z mike $".
+ * End of "$Id: ipp-support.c 10424 2012-04-23 17:26:57Z mike $".
  */
