@@ -1,9 +1,9 @@
 /*
- * "$Id: mark.c 9793 2011-05-20 03:49:49Z mike $"
+ * "$Id: mark.c 10424 2012-04-23 17:26:57Z mike $"
  *
  *   Option marking routines for CUPS.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -257,11 +257,13 @@ cupsMarkOptions(
       * Map sides to duplex option...
       */
 
-      if (!strcmp(sides, "one-sided"))
+      if (!strcmp(sides, "one-sided") && cache->sides_1sided)
         ppd_mark_option(ppd, cache->sides_option, cache->sides_1sided);
-      else if (!strcmp(sides, "two-sided-long-edge"))
+      else if (!strcmp(sides, "two-sided-long-edge") &&
+               cache->sides_2sided_long)
         ppd_mark_option(ppd, cache->sides_option, cache->sides_2sided_long);
-      else if (!strcmp(sides, "two-sided-short-edge"))
+      else if (!strcmp(sides, "two-sided-short-edge") &&
+               cache->sides_2sided_short)
         ppd_mark_option(ppd, cache->sides_option, cache->sides_2sided_short);
     }
   }
@@ -573,7 +575,7 @@ ppdMarkOption(ppd_file_t *ppd,		/* I - PPD file record */
  *
  * Options are returned from all groups in ascending alphanumeric order.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 ppd_option_t *				/* O - First option or @code NULL@ */
@@ -591,7 +593,7 @@ ppdFirstOption(ppd_file_t *ppd)		/* I - PPD file */
  *
  * Options are returned from all groups in ascending alphanumeric order.
  *
- * @since CUPS 1.2/Mac OS X 10.5@
+ * @since CUPS 1.2/OS X 10.5@
  */
 
 ppd_option_t *				/* O - Next option or @code NULL@ */
@@ -1095,5 +1097,5 @@ ppd_mark_option(ppd_file_t *ppd,	/* I - PPD file */
 
 
 /*
- * End of "$Id: mark.c 9793 2011-05-20 03:49:49Z mike $".
+ * End of "$Id: mark.c 10424 2012-04-23 17:26:57Z mike $".
  */
