@@ -1,9 +1,9 @@
 /*
- * "$Id: cupsd.h 9470 2011-01-11 07:05:58Z mike $"
+ * "$Id: cupsd.h 10490 2012-05-21 17:40:22Z mike $"
  *
  *   Main header file for the CUPS scheduler.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -18,11 +18,10 @@
  * Include necessary headers.
  */
 
-#include <cups/http-private.h>
-#include <cups/string.h>
-#include <stdlib.h>
+#include <cups/cups-private.h>
+#include <cups/file-private.h>
+
 #include <limits.h>
-#include <errno.h>
 #include <time.h>
 #include <signal.h>
 #include <fcntl.h>
@@ -38,13 +37,7 @@
 #  include <unistd.h>
 #endif /* WIN32 */
 
-#include <cups/array.h>
-#include <cups/cups.h>
 #include "mime.h"
-#include <cups/http.h>
-#include <cups/ipp.h>
-#include <cups/i18n.h>
-#include <cups/debug.h>
 
 #if defined(HAVE_CDSASSL)
 #  include <CoreFoundation/CoreFoundation.h>
@@ -89,12 +82,10 @@ extern const char *cups_hstrerror(int);
  * Defaults...
  */
 
-#define DEFAULT_HISTORY		1	/* Preserve job history? */
-#define DEFAULT_FILES		0	/* Preserve job files? */
+#define DEFAULT_HISTORY		INT_MAX	/* Preserve job history? */
+#define DEFAULT_FILES		86400	/* Preserve job files? */
 #define DEFAULT_TIMEOUT		300	/* Timeout during requests/updates */
 #define DEFAULT_KEEPALIVE	30	/* Timeout between requests */
-#define DEFAULT_INTERVAL	30	/* Interval between browse updates */
-#define DEFAULT_CHARSET		"utf-8"	/* Default charset */
 
 
 /*
@@ -125,6 +116,7 @@ extern const char *cups_hstrerror(int);
 #include "printers.h"
 #include "classes.h"
 #include "job.h"
+#include "colorman.h"
 #include "conf.h"
 #include "banners.h"
 #include "dirsvc.h"
@@ -180,70 +172,70 @@ VAR int			Launchd		VALUE(0);
 					/* Running from launchd */
 #endif /* HAVE_LAUNCH_H */
 
-#if defined(__APPLE__) && defined(HAVE_DLFCN_H)
-typedef int (*PSQUpdateQuotaProcPtr)(const char *printer, const char *info, 
-                                     const char *user, int nPages, int options);
-VAR PSQUpdateQuotaProcPtr PSQUpdateQuotaProc
-					VALUE(0);
-					/* Apple PrintService quota function */
-#endif /* __APPLE__ && HAVE_DLFCN_H */
-
-
-
 
 /*
  * Prototypes...
  */
 
-extern void	cupsdCheckProcess(void);
-extern void	cupsdClearString(char **s);
-extern void	cupsdHoldSignals(void);
-extern void	cupsdReleaseSignals(void);
-extern void	cupsdSetString(char **s, const char *v);
-extern void	cupsdSetStringf(char **s, const char *f, ...)
-#ifdef __GNUC__
-__attribute__ ((__format__ (__printf__, 2, 3)))
-#endif /* __GNUC__ */
-;
-extern void	cupsdStartServer(void);
-extern void	cupsdStopServer(void);
-extern void	cupsdClosePipe(int *fds);
-extern int	cupsdOpenPipe(int *fds);
+/* env.c */
+extern void		cupsdInitEnv(void);
+extern int		cupsdLoadEnv(char *envp[], int envmax);
+extern void		cupsdSetEnv(const char *name, const char *value);
+extern void		cupsdSetEnvf(const char *name, const char *value, ...)
+			__attribute__ ((__format__ (__printf__, 2, 3)));
+extern void		cupsdUpdateEnv(void);
 
-extern void	cupsdInitEnv(void);
-extern int	cupsdLoadEnv(char *envp[], int envmax);
-extern void	cupsdSetEnv(const char *name, const char *value);
-extern void	cupsdSetEnvf(const char *name, const char *value, ...)
-#ifdef __GNUC__
-__attribute__ ((__format__ (__printf__, 2, 3)))
-#endif /* __GNUC__ */
-;
-extern void	cupsdUpdateEnv(void);
+/* file.c */
+extern void		cupsdCleanFiles(const char *path, const char *pattern);
+extern int		cupsdCloseCreatedConfFile(cups_file_t *fp,
+			                          const char *filename);
+extern void		cupsdClosePipe(int *fds);
+extern cups_file_t	*cupsdCreateConfFile(const char *filename, mode_t mode);
+extern cups_file_t	*cupsdOpenConfFile(const char *filename);
+extern int		cupsdOpenPipe(int *fds);
+extern int		cupsdRemoveFile(const char *filename);
 
-extern void	*cupsdCreateProfile(int job_id);
-extern void	cupsdDestroyProfile(void *profile);
-extern int	cupsdEndProcess(int pid, int force);
-extern const char *cupsdFinishProcess(int pid, char *name, int namelen,
-		                      int *job_id);
-extern int	cupsdStartProcess(const char *command, char *argv[],
-				  char *envp[], int infd, int outfd,
-				  int errfd, int backfd, int sidefd,
-				  int root, void *profile, cupsd_job_t *job,
-				  int *pid);
+/* main.c */
+extern int		cupsdAddString(cups_array_t **a, const char *s);
+extern void		cupsdCheckProcess(void);
+extern void		cupsdClearString(char **s);
+extern void		cupsdFreeStrings(cups_array_t **a);
+extern void		cupsdHoldSignals(void);
+extern char		*cupsdMakeUUID(const char *name, int number,
+				       char *buffer, size_t bufsize);
+extern void		cupsdReleaseSignals(void);
+extern void		cupsdSetString(char **s, const char *v);
+extern void		cupsdSetStringf(char **s, const char *f, ...)
+			__attribute__ ((__format__ (__printf__, 2, 3)));
 
-extern int	cupsdAddSelect(int fd, cupsd_selfunc_t read_cb,
-		               cupsd_selfunc_t write_cb, void *data);
-extern int	cupsdDoSelect(long timeout);
+/* process.c */
+extern void		*cupsdCreateProfile(int job_id);
+extern void		cupsdDestroyProfile(void *profile);
+extern int		cupsdEndProcess(int pid, int force);
+extern const char	*cupsdFinishProcess(int pid, char *name, int namelen,
+					    int *job_id);
+extern int		cupsdStartProcess(const char *command, char *argv[],
+					  char *envp[], int infd, int outfd,
+					  int errfd, int backfd, int sidefd,
+					  int root, void *profile,
+					  cupsd_job_t *job, int *pid);
+
+/* select.c */
+extern int		cupsdAddSelect(int fd, cupsd_selfunc_t read_cb,
+			               cupsd_selfunc_t write_cb, void *data);
+extern int		cupsdDoSelect(long timeout);
 #ifdef CUPSD_IS_SELECTING
-extern int	cupsdIsSelecting(int fd);
+extern int		cupsdIsSelecting(int fd);
 #endif /* CUPSD_IS_SELECTING */
-extern void	cupsdRemoveSelect(int fd);
-extern void	cupsdStartSelect(void);
-extern void	cupsdStopSelect(void);
+extern void		cupsdRemoveSelect(int fd);
+extern void		cupsdStartSelect(void);
+extern void		cupsdStopSelect(void);
 
-extern int	cupsdRemoveFile(const char *filename);
+/* server.c */
+extern void		cupsdStartServer(void);
+extern void		cupsdStopServer(void);
 
 
 /*
- * End of "$Id: cupsd.h 9470 2011-01-11 07:05:58Z mike $".
+ * End of "$Id: cupsd.h 10490 2012-05-21 17:40:22Z mike $".
  */
