@@ -1,9 +1,9 @@
 /*
- * "$Id: lp.c 9636 2011-03-21 22:02:00Z mike $"
+ * "$Id: lp.c 10494 2012-05-21 22:39:01Z mike $"
  *
  *   "lp" command for CUPS.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -46,7 +46,7 @@ main(int  argc,				/* I - Number of command-line arguments */
   int		i, j;			/* Looping vars */
   int		job_id;			/* Job ID */
   char		*printer,		/* Printer name */
-		*instance,		/* Instance name */ 
+		*instance,		/* Instance name */
 		*val,			/* Option value */
 		*title;			/* Job title */
   int		priority;		/* Job priority (1-100) */
@@ -121,7 +121,7 @@ main(int  argc,				/* I - Number of command-line arguments */
               cupsSetUser(argv[i]);
 	    }
 	    break;
-	    
+
         case 'c' : /* Copy to spool dir (always enabled) */
 	    break;
 
@@ -605,7 +605,6 @@ main(int  argc,				/* I - Number of command-line arguments */
     const char		*format;	/* Document format */
     ssize_t		bytes;		/* Bytes read */
 
-
     if (cupsGetOption("raw", num_options, options))
       format = CUPS_FORMAT_RAW;
     else if ((format = cupsGetOption("document-format", num_options,
@@ -623,11 +622,17 @@ main(int  argc,				/* I - Number of command-line arguments */
     {
       _cupsLangPrintf(stderr, _("%s: Error - unable to queue from stdin - %s."),
 		      argv[0], httpStatus(status));
+      cupsFinishDocument(CUPS_HTTP_DEFAULT, printer);
+      cupsCancelJob2(CUPS_HTTP_DEFAULT, printer, job_id, 0);
       return (1);
     }
 
     if (cupsFinishDocument(CUPS_HTTP_DEFAULT, printer) != IPP_OK)
-      job_id = 0;
+    {
+      _cupsLangPrintf(stderr, "%s: %s", argv[0], cupsLastErrorString());
+      cupsCancelJob2(CUPS_HTTP_DEFAULT, printer, job_id, 0);
+      return (1);
+    }
   }
 
   if (job_id < 1)
@@ -719,5 +724,5 @@ set_job_attrs(const char    *command,	/* I - Command name */
 
 
 /*
- * End of "$Id: lp.c 9636 2011-03-21 22:02:00Z mike $".
+ * End of "$Id: lp.c 10494 2012-05-21 22:39:01Z mike $".
  */
